% Das Programm berechnet die in der Aufgabe gefordeten Daten, gibt eine
% grafische Veranschaulichung aus und stellt dem Anwender zur weiteren
% fr die spaetere Nutzung die x-, y-, und z-Koordinaten in einem Zeilenvektor
% bereit. Die jeweiligen Punkte lassen sich bei Bedarf durch
%
% P = [x(k) y(k) z(k)]'
%
% als Spaltenvektor konstruieren.
%
% Die Koordinatenprojektion fuer die Kurve erhaelt man jeweils durch Eingabe
% von:
%
% plot(x, y)
% plot(y, z)
% plot(x, z)
%
% Bei Bedarf kann innerhalb der grafischen Veranschaulichung von Flaeche und
% Verlauf des Lichtstrahls der Lichtkegel zusaetzlich (in gelb) ausgegeben 
% werden mit:
%
% hold on;
% plot3(X, Y, Zk, 'y');

% Autor: Oliver Tacke
% System: Ubuntu Linux 8.04, Octave 3.0.0.9

% Erst einmal aufraeumen
clear all;

% Schrittweite zum Zaehlen von phi
step = 2*pi/200;

% Farbe Grau mit Verlauf
colGrey = [0.6 0.6 0.6; 0.7 0.7 0.7; 0.8 0.8 0.8];

% Zaehlervariable
k = 1;

% Genauigkeit fr die Nullstellenberechnung
tol = 1e-15;

% "linke" und "rechte Seite als Startwerte fr das Intervallhalbierungsverfahren
left = 0.3;
right = 2.2;

% Startmeldung %
printf("\nStarte Berechnung, bitte warten...\n");

% Innerhalb der Schleife werden die Schnittpunkte zwischen
% Lichtkegel und Oberflaeche berechnet. Um bei der Ausgabe der Kurven
% keine "Luecken" zu haben, luft der Winkel phi von 0 bis 2pi (inklusive).

for (phi = 0:step:2*pi)
	% Berechnung der jeweiligen Nullstelle fr das aktuelle phi
	r0 = nullstelle("nullfunktion", phi, left, right, tol);

	% Berechnung der Koordinaten des Schnitts
	x(k) = cos(phi)*r0;
	y(k) = sin(phi)*r0;
	z(k) = 1-pi/4*r0;
	k = k + 1;
end

% Erzeugung der Hilfsmatrizen fr die Ausgabe
X = ones(size(x))'*x;
Y = y'*ones(size(y));
Ys = Y';

% Berechnung der z-Koordinate fr die Flaeche (Zf), den Lichtkegel (Zk) und den
% Schnitt (Zs).
Zf = -cos(X.*Y.^3)./sqrt(3.+sin(X).*cos(Y));
Zk = 1.-pi/4.*sqrt(X.*X.+Y.*Y);
Zs = 1.-pi/4.*sqrt(X.*X.+Ys.*Ys);

% Ausgabe von
% - Flaeche in grau
% - Schnitt in rot
% - Position der Lichtquelle in rot

clf;
hold on;

colormap(colGrey);
mesh(X, Y, Zf);
% Als Alternative koennte man z. B. eine Oberflaechendarstellung waehlen
%surf(X, Y, Zf);
%shading interp;

plot3(0, 0, 1, 'ro');
plot3(X, Ys, Zs, 'r');

% Bei Bedarf kann der Lichtkegel zusaetzlich (in schwarz) ausgegeben werden
% hold on;
% plot3(X, Y, Zk, '0');

hold off;

% Ausgabe der Schlussmeldung
printf("Berechnung abgeschlossen\n\n");
printf("Die Koordinatenprojektion fuer die Kurve erhaelt man jeweils durch...
 Eingabe\n");
printf("von:\n\n");
printf("  plot(x, y)\n");
printf("  plot(y, z)\n");
printf("  plot(x, z)\n\n");

printf("Bei Bedarf kann innerhalb der grafischen Veranschaulichung von...
 Flaeche und\n");
printf("Verlauf des Lichtstrahls der Lichtkegel zusaetzlich (in gelb)...
 ausgegeben \n");
printf("werden mit:\n\n");
printf("  hold on\n");
printf("  plot3(X, Y, Zk, 'y')\n\n");
